import http
import logging
from typing import Optional

from flask import g
from flask_restplus import Resource, fields

from app.libs.flask_rest_plus_additions import Namespace
from app.services.scheduler.result_containers.planner_result_schedule import PlannerResultSchedule
from app.services.scheduler.match_matrix_builder import MatchMatrix

logger = logging.getLogger(__name__)
ns = Namespace("debug", description="Endpoint for interaction with the print job queue")

matchResultModel = ns.model("DebugMatchResult", {
    "type": fields.String(required=False, description="The field type enum"),
    "origin": fields.String(required=False, description="The origin of the match result"),
    "debug_message": fields.String(required=True, description="Message string of the match result"),
    "score": fields.Integer(requiered=True, description="low is good!"),
    "severity": fields.Integer(requiered=True, description="enum values for different severities")
})

printerModel = ns.model("MinimalPrinter", {
    "uuid": fields.String(required=False, description="The identifier"),
    "unique_name": fields.String(required=True, description="The printer's unique name"),
    "friendly_name": fields.String(required=True, description="Human readable name")
})

printJobModel = ns.model("MinimalPrintJob", {
    "uuid": fields.String(required=True, description="Uniquely identifies the print job"),
    "name": fields.String(required=True, description="The human readable filename")
})

matchResultPerPrinter = ns.model("MatchResultPerPrinter", {
    "printer": fields.Nested(printerModel),
    "results": fields.List(fields.Nested(matchResultModel))
})

matchResultPerJob = ns.model("MatchResultPerJob", {
    "print_job": fields.Nested(printJobModel),
    "results": fields.List(fields.Nested(matchResultPerPrinter))
})

schedule = ns.model("Schedule", {
    "printer": fields.Nested(printerModel),
    "jobs": fields.List(fields.Nested(printJobModel))
})


@ns.route("/match_matrix/<int:nr>")
class DebugMatchMatrix(Resource):
    """
    MatchMatrix debugging
    """
    @ns.doc("get_match_matrix")
    @ns.marshal_list_with(matchResultPerJob)
    def get(self, nr: int):
        """
        Shows a list of all the last job <-> printer matcher from the scheduler
        """
        matrix = None  # type: Optional[MatchMatrix]
        if nr == 0:
            matrix = g.print_job_scheduler().debug_get_current_match_matrix()
        else:
            matrix = g.print_job_scheduler().debug_get_match_matrix_history(-1 * int(nr))
        if matrix is not None:
            return matrix.get_match_result_per_job()

        return '', http.client.NO_CONTENT


@ns.route("/schedule/<int:nr>")
class DebugPlannerSchedule(Resource):
    """
    MatchMatrix debugging
    """
    @ns.doc("get_schedule")
    @ns.marshal_list_with(schedule)
    def get(self, nr: int):
        """
        Shows a list of all the last job <-> printer matcher from the scheduler
        """
        schedule = None  # type: Optional[PlannerResultSchedule]
        if nr == 0:
            schedule = g.print_job_scheduler().debug_get_current_schedule()
        else:
            schedule = g.print_job_scheduler().debug_get_schedule_history(-1 * int(nr))
        if schedule is not None:
            return schedule.debug_get_schedule()

        return '', http.client.NO_CONTENT
