from flask import request, g
import logging

from flask_restplus import Resource, fields
import http.client

from app.libs.flask_rest_plus_additions import Namespace
from um3_commons.registry.registryFile import RegistryFile

logger = logging.getLogger(__name__)

ns = Namespace("settings", description="Endpoint for at runtime configuration")

model = ns.model("Settings", {
    "payload": fields.Raw(
        description="Type depends on the identifier being passed in.",
        required=True
    )
})

configuration_settings = {
    "data_collection_enabled": {
        "key": "ALLOW_ANONYNMOUS_DATA_COLLECTION",
        "type": bool,
        "method": RegistryFile.setAsBoolean
    }
}


@ns.route('/<string:identifier>')
class Settings(Resource):
    def get(self, identifier):
        registry_file = g.registry_file()  # type: RegistryFile

        if identifier not in configuration_settings:
            return http.client.METHOD_NOT_ALLOWED

        return registry_file.get(configuration_settings[identifier]["key"])

    @ns.expect(model)
    def put(self, identifier):
        registry_file = g.registry_file()  # type: RegistryFile

        if identifier not in configuration_settings:
            return http.client.METHOD_NOT_ALLOWED

        json = request.get_json(force=True)
        if isinstance(json["payload"], configuration_settings[identifier]["type"]):
            # call the setter method corresponding to the identifier and key
            configuration_settings[identifier]["method"](
                registry_file,
                configuration_settings[identifier]["key"],
                json["payload"]
            )

            return json["payload"]
        else:
            return "Type mismatch", http.client.BAD_REQUEST
