from flask import Blueprint
from flask_restplus import Api

from app.apis.print_jobs import ns as print_jobs
from app.apis.printers import ns as printers
from app.apis.settings import ns as settings
from app.apis.debug import ns as debug
from app.errors.invalid_usage import InvalidUsage

URL_PREFIX = "/cluster-api/v1"

mod = Blueprint("api", __name__, url_prefix=URL_PREFIX)

api = Api(
    mod,
    title="Ultimaker - Print Cluster API",
    version="0.1",
    description="This API exposes endpoints to interact with the print cluster software",
)

api.add_namespace(print_jobs)
api.add_namespace(printers)
api.add_namespace(settings)
api.add_namespace(debug)


@api.errorhandler(InvalidUsage)
def handle_invalid_usage(error):
    return {"message": error.message}, error.status_code

