import os
import logging
import shutil

from flask import current_app as app
from werkzeug.utils import secure_filename
from werkzeug.datastructures import FileStorage

from app.errors.invalid_usage import InvalidUsage

logger = logging.getLogger(__name__)


class FileUploadHandler:

    def __init__(self, uuid: str, file: FileStorage) -> None:
        self.__uuid = uuid
        self.__file = file

        if not self.__is_allowed_filetype():
            raise InvalidUsage("File not allowed!")

    def get_filename(self) -> str:
        return secure_filename(self.__file.filename)

    def get_filepath(self) -> str:
        return os.path.join(app.config["SPOOL_DIR"], self.__uuid, self.get_filename())

    def save_to_disk(self) -> None:
        filepath = self.get_filepath()
        logger.debug("Saving file [%s]..." % filepath)
        os.makedirs(os.path.dirname(filepath), exist_ok=True)
        self.__file.save(filepath)

    def __is_allowed_filetype(self) -> bool:
        return os.path.splitext(self.__file.filename)[1].lower() in (".gcode", ".gz")

    @staticmethod
    def clear_all_uploaded_files() -> None:
        logger.info("Clearing all uploaded files")
        # Delete all the uploaded files, without deleting the spool directory itself.
        for path in os.listdir(app.config["SPOOL_DIR"]):
            shutil.rmtree(os.path.join(app.config["SPOOL_DIR"], path), ignore_errors=True)
