
import pyrsistent
from pyrsistent import PVector
from typing import TypeVar, Generic, Iterable, Iterator, Hashable, Sized

T = TypeVar('T')


class pvector(Generic[T], Iterable, Sized, Hashable):

    def __init__(self) -> None:
        self.__pvector = pyrsistent.pvector()

    def __len__(self) -> int:
        return self.__pvector.__len__()

    def __getitem__(self, index: int) -> T:
        return self.__pvector.__getitem__(index)

    def __add__(self, other: 'pvector[T]') -> 'pvector[T]':
        return self.__pvector.__add__(other)

    def __repr__(self) -> str:
        return self.__pvector.__repr__()

    def __str__(self) -> str:
        return self.__pvector.__str__()

    def __ne__(self, other: object) -> bool:
        return self.__pvector.__ne__(other)

    def __eq__(self, other: object) -> bool:
        return self.__pvector.__eq__(other)

    def __gt__(self, other: 'pvector[T]') -> int:
        return self.__pvector.__gt__(other)

    def __lt__(self, other: 'pvector[T]') -> int:
        return self.__pvector.__lt__(other)

    def __ge__(self, other: 'pvector[T]') -> int:
        return self.__pvector.__ge__(other)

    def __le__(self, other: 'pvector[T]') -> int:
        return self.__pvector.__le__(other)

    def __mul__(self, times: int) -> 'pvector[T]':
        return self.__pvector.__mul__(times)

    __rmul__ = __mul__

    def __hash__(self) -> int:
        return self.__pvector.__hash__()

    def set(self, i: int, val: T) -> 'pvector[T]':
        return self.__pvector.set(i, val)

    def append(self, item: T) -> 'pvector[T]':
        return self.__pvector.append(item)

    def extend(self, obj: 'pvector[T]') -> 'pvector[T]':
        return self.__pvector.extend(obj)

    def count(self, value: T) -> int:
        return self.__pvector.count(value)

    def delete(self, index: int, stop: int=None) -> 'pvector[T]':
        return self.__pvector.delete(index, stop)

    def remove(self, value: T) -> 'pvector[T]':
        return self.__pvector.remove(value)

    def __iter__(self) -> Iterator[T]:
        return self.__pvector.__iter__()


PVector.register(pvector)
