from typing import Dict, Any
from datetime import datetime
# sample json from the api, to be interpreted per item in the array
# [
#      {
#        "message": "A maintenance action was performed on Lubricate_axles by SomeOne.",
#        "parameters": [
#          "Lubricate_axles",
#          "SomeOne"
#        ],
#        "time": "2017-08-23T16:42:53",
#        "type_id": 4
#      },
#      {
#        "message": "A maintenance action was performed on Lubricate_axles by Robin.",
#        "parameters": [
#          "Lubricate_axles",
#          "Robin"
#        ],
#        "time": "2017-08-23T16:34:31",
#        "type_id": 4
#      }
# ]


class Maintenance:
    # This is the type_id field for maintenance events used by opinicus.
    type_id = 4

    def __init__(self, data: Dict[str, Any]) -> None:
        self.string = str(data["message"])          # type: str
        self.action = str(data["parameters"][0])     # type: str
        self.mechanic = str(data["parameters"][1])   # type: str
        self.timestamp = datetime.strptime(data["time"], "%Y-%m-%dT%H:%M:%S")  # type: datetime

    def __str__(self) -> str:
        return "Maintenance({})".format(self.string)

    def __repr__(self) -> str:
        return str(self)
