import logging
import logging.handlers
import sys


class ClusterFilter(logging.Filter):
    EXCLUDE_NAMES = {
        'requests.packages.urllib3.connectionpool',
        'urllib3.connectionpool',  # On the printer, apparently
        # 'werkzeug',
    }

    def filter(self, record):
        return record.name not in self.EXCLUDE_NAMES


def setup_logging(
        console_level=logging.INFO,
        console_formatter='%(asctime)s - %(levelname).3s - %(name)s:%(lineno)s - %(message)s'):
    """ Set up root logger that logs everything. """
    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    console_handler = logging.StreamHandler(sys.stderr)
    console_handler.setLevel(console_level)
    formatter = logging.Formatter(console_formatter, datefmt='%H:%M:%S')
    console_handler.setFormatter(formatter)
    console_handler.addFilter(ClusterFilter())
    root_logger.addHandler(console_handler)

    # Enable to selectively enable debug logging for components:
    #logging.getLogger("app.libs.printer_discovery").setLevel(logging.DEBUG)
    #logging.getLogger("app.services.printer_monitor_service").setLevel(logging.DEBUG)
    #logging.getLogger("app.models.printer").setLevel(logging.DEBUG)

    return root_logger
