from threading import RLock
from typing import List, Optional, Iterator

from app.models.extruder_train import ExtruderTrain


class Configuration:
    def __init__(self) -> None:
        self.__config_list = []  # type: List[ExtruderTrain]
        self.__lock = RLock()

    def is_empty(self) -> bool:
        return not bool(self.__config_list)

    def get(self, index: int) -> Optional[ExtruderTrain]:
        for et in self.__config_list:
            if et.extruder_index == index:
                return et
        return None

    def update(self, object: ExtruderTrain) -> None:
        with self.__lock:
            for list_index, et in enumerate(self.__config_list):
                if et.extruder_index == object.extruder_index:
                    del(self.__config_list[list_index])
            self.__config_list.append(object)

    def __iter__(self) -> Iterator[ExtruderTrain]:
        return iter(self.__config_list)

    def __len__(self):
        return len(self.__config_list)
