from typing import Any
from um3_commons.math.axisAlignedBox import AxisAlignedBox
from um3_commons.math.vector3 import Vector3


class MachineVariant:
    ULTIMAKER_3 = "Ultimaker 3"
    ULTIMAKER_3_EXTENDED = "Ultimaker 3 Extended"

    # TODO: This data is currently copied from the machine json files in opinicus, as there is no API/mechanism
    # to get to this data.
    __PRINT_VOLUME = {
        ULTIMAKER_3: AxisAlignedBox(Vector3(0, 0, 0), Vector3(215, 215, 210)),
        ULTIMAKER_3_EXTENDED: AxisAlignedBox(Vector3(0, 0, 0), Vector3(215, 215, 310)),
    }

    __EXTRUDER_COUNT = {
        ULTIMAKER_3: 2,
        ULTIMAKER_3_EXTENDED: 2
    }

    def __init__(self, variant):
        self.variant = variant

    @property
    def print_volume(self) -> AxisAlignedBox:
        """
        Return total print volume in mm.
        """
        return MachineVariant.__PRINT_VOLUME.get(self.variant, AxisAlignedBox(Vector3(0, 0, 0), Vector3(0, 0, 0)))

    @property
    def extruder_count(self) -> int:
        """
        Return nr of extruders.
        """
        return MachineVariant.__EXTRUDER_COUNT.get(self.variant, 0)

    def __eq__(self, other: Any):
        if isinstance(other, MachineVariant):
            return self.variant == other.variant
        return False

    def __str__(self):
        return self.variant
