from typing import Sequence

from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult
from app.services.scheduler.result_containers.match_results_containers import MatchMatrix
from app.services.scheduler.result_containers.match_results_containers import MatchResultListPerJob
from app.services.scheduler.result_containers.match_results_containers import MatchResultListPerPrinter
from app.services.scheduler.match_rule import MatchRule


# This class builds a 2x2 matrix of a list of match results
# An empty list means a perfect match
class MatchMatrixBuilder:
    def __init__(self, match_rules: Sequence[MatchRule]) -> None:
        self._match_rules = match_rules

    def build(self, print_jobs: Sequence[PrintJob], printers: Sequence[Printer]) -> MatchMatrix:
        matrix = MatchMatrix()
        for job in print_jobs:
            job_results = MatchResultListPerJob(job)
            for printer in printers:
                printer_results = MatchResultListPerPrinter(printer)
                for match_rule in self._match_rules:
                    try:
                        result = match_rule.apply(printer, job)
                    except Exception as e:
                        result = IncompatibleMatchResult(
                            origin=match_rule,
                            debug_message="Exception in matcher: " + str(e),
                            translation_key="exception"
                        )
                    printer_results.append(result)

                job_results.append(printer_results)
            matrix.append(job_results)
        return matrix
