from app.models.machine_variant import MachineVariant
from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import MatchResult, IncompatibleMatchResult


class DoesPrinterMatchTargetFamilyRule(MatchRule):
    FAMILY = [MachineVariant.ULTIMAKER_3, MachineVariant.ULTIMAKER_3_EXTENDED]

    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        if not (printer.machine_variant.variant in self.FAMILY and print_job.machine_variant.variant in self.FAMILY):
            return IncompatibleMatchResult(
                origin=self,
                debug_message="Not a compatible UM3 type printer",
                translation_key="family_incompatible"
            )
        return None
