from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import ConfigurationChangeMatchResult
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult
from app.services.scheduler.result_containers.match_results import MatchResult


class IsHotendCartridgeCompatibleMatchRule(MatchRule):
    def __init__(self, printcore_index: int) -> None:
        self.__printcore_index = printcore_index

    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        if printer.configuration.is_empty():
            return IncompatibleMatchResult(
                origin=self,
                debug_message="The printer does not have any configuration yet",
                translation_key="loading_config"
            )

        printer_config = printer.configuration.get(self.__printcore_index)
        job_config = print_job.configuration.get(self.__printcore_index)

        if job_config is None:
            return None

        # TODO: discuss using BB nozzle as if it is AA (it'll just ooze a bit more, maybe only warning?)
        if printer_config is None or printer_config.print_core_id != job_config.print_core_id:
            return ConfigurationChangeMatchResult(
                change=ConfigurationChangeMatchResult.ChangeSet(
                    type_of_change="print_core_change",
                    index=self.__printcore_index,
                    origin_id=printer_config.print_core_id if printer_config is not None else None,
                    target_id=job_config.print_core_id),
                origin=self,
                debug_message="hotend cartridge type mismatch on slot {}".format(self.__printcore_index),
                translation_key="print_core_change"
            )
        return None
