from app.libs.ultimaker3.material import Material
from app.models.extruder_train import ExtruderTrain
from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import ConfigurationChangeMatchResult
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult
from app.services.scheduler.result_containers.match_results import MatchResult


class IsMaterialPresentMatchRule(MatchRule):
    def __init__(self, printcore_index: int) -> None:
        self.__printcore_index = printcore_index

    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        if printer.configuration.is_empty():
            return IncompatibleMatchResult(
                origin=self,
                debug_message="The printer configuration is empty",
                translation_key="loading_config"
            )

        printer_config = printer.configuration.get(self.__printcore_index)  # type: ExtruderTrain
        job_config = print_job.configuration.get(self.__printcore_index)  # type: ExtruderTrain

        if job_config is None:
            return None

        if printer_config is None or printer_config.material == Material.EMPTY_MATERIAL:
            return ConfigurationChangeMatchResult(
                change=ConfigurationChangeMatchResult.ChangeSet(
                    type_of_change="material_insert",
                    index=self.__printcore_index,
                    origin_id=printer_config.material_guid if printer_config is not None else None,
                    target_id=job_config.material_guid),
                origin=self,
                debug_message="Material mismatch on slot {}: printer has no material, job expects {}".format(
                    self.__printcore_index, job_config.material.pretty_string),
                translation_key="material_insert"
            )

        return None
