from app.constants.printer_status import PrinterStatus
from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult, MatchResult, Severity


class IsPrinterReachable(MatchRule):
    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        if printer.status == PrinterStatus.UNREACHABLE:
            return IncompatibleMatchResult(
                origin=self,
                debug_message="Printer cannot be reached",
                translation_key="unreachable_printer",
                severity=Severity.USER_FIXABLE
            )
        return None
