from typing import List, Tuple

from app.models.print_job import PrintJob
from app.models.printer import Printer
from app.services.scheduler.match_rule import MatchRule, Optional
from app.services.scheduler.result_containers.match_results import IncompatibleMatchResult, MatchResult, Severity


# This rule exists to prevent the printer from starting without both cartridges present.
# The printer will not accept a job at that time because airflow can't be guaranteed.
class PrinterIsMissingPrintCoresMatchRule(MatchRule):
    def apply(self, printer: Printer, print_job: PrintJob) -> Optional[MatchResult]:
        printcore_ids = []  # type: List[Tuple[int, str]]
        for i in range(printer.machine_variant.extruder_count):
            extruder_train = printer.configuration.get(i)
            if extruder_train is None:
                printcore_ids.append((i, ""))
            else:
                printcore_ids.append((i, extruder_train.print_core_id))

        for index, printcore_id in printcore_ids:
            if printcore_id == "":
                return IncompatibleMatchResult(
                    origin=self,
                    debug_message="Print Core {} is missing from Printer.".format(index),
                    translation_key="printer_is_missing_printcores",
                    severity=Severity.USER_FIXABLE
                )
        return None
