import os
import tempfile
import logging

logger = logging.getLogger(__name__)

app_root = os.path.join(os.path.dirname(__file__), "..")
database_directory = os.path.join(os.path.abspath(app_root), "db")


## This configuration is defined at build time.
class Config(object):
    CONFIGURATION_FILE_BASE_DIR = database_directory
    CONFIGURATION_FILE_NAME = "preferences.json"

    SECRET_KEY = "tobereplaced.."
    SPOOL_DIR = "uploads"

    PRINTER_IP = "192.168.2.127"
    PRINTER_CREDENTIAL_ID = ""
    PRINTER_CREDENTIAL_KEY = ""
    PRINTER_CREDENTIAL_AUTH_HEADER = ""

    LOG_LEVEL = logging.INFO

    TESTING = False


class ProdConfig(Config):
    ENV = "prod"

    SPOOL_DIR = "/var/spool/cluster"
    SQLALCHEMY_DATABASE_URI = os.path.join("sqlite:///var/lib/griffin/cluster", "database.db")
    EVENTSTORE_FILEPATH = os.path.join("/var/lib/griffin/cluster", "eventstore.db")
    CONFIGURATION_FILE_BASE_DIR = "/var/lib/griffin/cluster"


class DevConfig(Config):
    ENV = "dev"
    DEBUG = True
    MATERIALS_PATH = os.path.join(app_root, "misc", "fdm_materials")

    SQLALCHEMY_DATABASE_URI = os.path.join("sqlite:///" + database_directory, "database.db")
    EVENTSTORE_FILEPATH = os.path.join(database_directory, "eventstore.db")

    LOG_LEVEL = logging.DEBUG


class TestConfig(Config):
    db_file = tempfile.NamedTemporaryFile()
    logging.getLogger("factory").setLevel(logging.WARN)

    ENV = "test"
    DEBUG = True
    TESTING = True
    MATERIALS_PATH = os.path.join(app_root, "misc", "fdm_materials")

    LIVESERVER_PORT = 8943
    LIVESERVER_TIMEOUT = 10

    SQLALCHEMY_DATABASE_URI = "sqlite:///" + db_file.name
    SQLALCHEMY_ECHO = True
    EVENTSTORE_FILEPATH = os.path.join(database_directory, "testeventstore.db")


class BuildConfig(Config):
    ENV = "build"
    DEBUG = False
    MATERIALS_PATH = os.path.join(app_root, "misc", "fdm_materials")
    EVENTSTORE_FILEPATH = os.path.join(database_directory, "buildeventstore.db")


class Version():
    __JEDI_VERSION_FILE = "/etc/jedi_version"
    __GIT_VERSION = "jedi-master-2018.02.22-6-g17e3a7e"
    __PACKAGE_VERSION = "2018.02.27"

    def __init__(self):
        image_version = ""
        try:
            with open(self.__JEDI_VERSION_FILE, "r") as f:
                image_version = f.readline()
        except Exception:
            image_version = "Image version could not be read"

        if image_version.startswith("MOD-"):
            logger.warning("*** BASE IMAGE VERSION: %s (Note that this image has been modified after installation)",
                           image_version)
        else:
            logger.info("Image Version: %s", image_version)

        if self.__PACKAGE_VERSION == "9999.99.99":
            logger.info("Package Version %s -> Git Version %s", self.__PACKAGE_VERSION, self.__GIT_VERSION)
        else:
            logger.info("Package Version %s", self.__PACKAGE_VERSION)
