import datetime
import json

from typing import Dict, Any, Optional


class PersistentEvent(object):
    """Represents an event which is stored in the Event store."""

    def __init__(self, id: Optional[int], event_type: int, entity_uuid: str, boot_time: float = 0,
                 timestamp: datetime.datetime = None,
                 props_dict: Optional[Dict[str, Any]]=None) -> None:
        self.id = id
        self.boot_time = boot_time
        self.timestamp = timestamp
        self.event_type = event_type
        self.entity_uuid = entity_uuid
        self.props = props_dict

    @classmethod
    def from_row(cls, row) -> "PersistentEvent":
        properties = json.loads(row["properties"]) if row["properties"] is not None else None
        return PersistentEvent(id=row["id"], boot_time=row["boot_time"],
                               timestamp=row["timestamp"], event_type=row["event_type"],
                               entity_uuid=row["entity_uuid"], props_dict=properties)

    def __repr__(self) -> str:
        return ("{id: %s, boot_time: %f, timestamp: %s, event_type: %s, entity_uuid: %s, " +
                "props: %s}") % (self.id, self.boot_time, str(self.timestamp), self.event_type,
                                 self.entity_uuid, self.props)
