The avrprog python module currently contains two different main programmers:
* Stk500v2: This programmer needs to talk trough a serial port. This programs the AVR by talking to an STK500v2 compattible bootloader.
    This protocol is used by Arduino as well as the Ultimaker 2. Firmware for this bootloader is provided by the global package as well.
* AvrISP: This programmer directly talks to the internal AVR in-circuit-programming interface. It needs an SPI port as well as a reset line to work. And can program even empty chips.
    This doesn't require any bootloader to be present in the chip. However, this isn't as fast as the Stk500v2 programming.

All programmers provide the same high level interface:

class AbstractProgrammer:
    # Connect to the hardware. Returns false if it fails to connect.
    # @param target: The target string is programmer specific, and is used to identify the hardware device used for communication.
    # @return: bool, True if opening the hardware and connecting to the chip was succesful. False if this failed.
    def connect(self, target : string):
    
    # Program a hex file into the chip.
    # @param file: The filename of the firmware to program.
    # @return: bool, True if programming was successful, False if it failed.
    def program(self, file):
    
    # Close the connection to the hardware, and boot the chip into the firmware.
    # Without this call the chip can remain in bootloader/programming mode and never start.
    def close(self):
    
    # Set a callback function to be called when the program function is making progress.
    # @param callback: A callable which will be called with a single float in the range 0 to 1 to indicate the progress made on updating the firmware, where 1.0 is finished.
    # @param callback: None de-attaches the previously attached callback.
    def setProgressCallback(self, callback):

    # get the current firmware update progress
    # @return float between 0 and 1 that indicates the update progress
    def getProgress(self):

Programmers can be imported by:
    from ucFirmwareUpgrade.avr import AvrISP
    from ucFirmwareUpgrade.avr import Stk500v2
    from ucFirmwareUpgrade.avr import Multi

It is also possible to program from the commandline:
```console
    # python3 -m ucFirmwareUpgrade.avr.avrisp.avrISP <filename>
```
