from . import hardwareSpi
from . import softwareSpi
from .lowLevelISP import LowLevelISP
from ...common.firmwareUpdateBase import FirmwareUpdateBase

import logging

log = logging.getLogger(__name__.split(".")[-1])


class AvrISP(FirmwareUpdateBase):
    # Connect to the hardware. Returns false if it fails to connect.
    # @param target: The target to program. In the form of "[dev]:[path] [dev]:[path]", [dev] can be "spi", "softspi" and "reset".
    #                   Example: "softspi:/dev/gpio/avr reset:/dev/gpio/avr_reset"
    # @return: LowLevel hardware object, or None if the hardware could not be found.
    def _connectLowLevel(self, spi=None, softspi=None, reset=None, **kwargs):
        spi_hardware = None
        reset_hardware = None
        if spi:
            spi_hardware = hardwareSpi.SpiBus(spi)
        if softspi:
            spi_hardware = softwareSpi.SpiBus(softspi)
        if spi_hardware is None or reset is None:
            return None

        return LowLevelISP(spi_hardware, reset)


def main(argc, argv):

    if argc != 2:
        print("Invalid parameters, only pass a filename to flash")
        return 1

    filename = argv[1]

    if not os.path.isfile(filename):
        print("Supplied file is not a valid file:", filename)
        return 1

    avr_isp = AvrISP()
    if avr_isp.connect(softspi="/dev/gpio/avr", reset="/dev/gpio/avr_reset"):
        avr_isp.program(filename)

    avr_isp.close()

    return 0


if __name__ == "__main__":
    logging.basicConfig(format="%(asctime)-15s %(levelname)-8s %(name)-15s %(message)s", level=logging.DEBUG)

    import os
    import sys
    for arg in sys.argv[1:]:
        sys.exit(main(len(sys.argv), sys.argv))
