import os.path
import logging
import re
import io

from typing import Dict, List, TextIO

log = logging.getLogger(__name__.split(".")[-1])

__SYSFS_RFKILL_CLASS_PATH = "/sys/class/rfkill/"

def block(idx : int) -> None:
    log.info("Blocking rfkill {0}".format(idx))
    with __softBlockFile(idx) as f:
        f.write("1")

def unblock(idx : int) -> None:
    log.info("Unblocking rfkill {0}".format(idx))
    with __softBlockFile(idx) as f:
        f.write("0")

def canBlock(idx : int) -> bool:
    return os.path.isfile(__softBlocKPath(idx))

def info(idx : int) -> Dict[str, str]:
    base_path = __rfkillPath(idx)
    result =  {"index": str(idx)}
    for prop in {"name", "soft", "hard", "state", "type", "persistent"}:
        file_path = os.path.join(base_path, prop)

        try:
            with open(file_path) as f:
                result[prop] = f.read().strip()
        except FileNotFoundError:
            pass
        except IOError:
            pass
    return result

def list()-> List[Dict[str, str]]:
    return [info(idx) for idx in __listIndices()]

def __listIndices() -> List[int]:
    all_files = os.listdir(__SYSFS_RFKILL_CLASS_PATH)
    rfkill_indices = []
    for f in all_files:
        match = re.match(r"^rfkill([0-9]+)$", f)
        if match:
            rfkill_indices.append(int(match.group(1)))
    return rfkill_indices

def __rfkillPath(idx : int) -> str:
    return os.path.join(__SYSFS_RFKILL_CLASS_PATH, "rfkill{0}".format(idx))

def __softBlocKPath(idx : int) -> str:
    return os.path.join(__rfkillPath(idx), "soft")

def __softBlockFile(rfkillIdx : int) -> TextIO:
    return open(__softBlocKPath(rfkillIdx), "w")