from griffin.datatypes.printJob import PrintJob
from griffin.history.alchemy import session_scope, PrintJobRecord


## @brief Owner of the database that stored all the jobs. It acts as the only entry point to it.
class JobHistoryBackend():
    def __init__(self):
        pass

    ## @brief Add a PrintJob to the database.
    # @param job, PrintJob.
    def addPrintJob(self, job):
        assert isinstance(job, PrintJob)
        with session_scope() as session:
            _job = PrintJobRecord(
                uuid=job.uuid,
                name=job.name,
                source=job.source,
                reprint_original_uuid=job.reprint_original_uuid,
                time_elapsed=job.time_elapsed,
                datetime_started=job.datetime_started,
                datetime_finished=job.datetime_finished,
                datetime_cleaned=job.datetime_cleaned,
                result=job.result,
                time_estimated=job.time_estimated
            )
            session.add(_job)

    ## @brief find a PrintJob by its UUID
    # @param uuid, uuid.UUID the ID of the PrintJob that must be found
    # @returns a _printJob (which can be used just like a normal PrintJob)
    def getJobByUuid(self, uuid):
        with session_scope() as session:
            result = session.query(PrintJobRecord).get(str(uuid))
            return None if result is None else result.asNative()

    ## @brief Return all jobs, but with a limit of up to jobCount _PrintJobs
    # @param offset, int: the offset from the end of the job list to start returning jobs. 0 is include the latest job.
    # @param jobCount, int: how many jobs to return.
    # @returns a list of _PrintJobs. These are sorted with the newest first
    def getAllJobs(self, offset, jobCount):
        with session_scope() as session:
            _jobs = session.query(PrintJobRecord).order_by(PrintJobRecord.datetime_started.desc()).offset(offset).limit(jobCount)
            return [_job.asNative() for _job in _jobs]

    def removeAllJobs(self):
        with session_scope() as session:
            session.query(PrintJobRecord).delete(synchronize_session=False)
