from griffin.interface.http.exposedItems.httpExposedObject import HttpExposedObject
from griffin.interface.http.flaskResponses import FlaskResponses

from griffin import dbusif

import logging
log = logging.getLogger(__name__.split(".")[-1])


## @brief Blink is a special case of an HttpExposedObject, which accepts POSTs that specify a frequency and a count
class Blink(HttpExposedObject):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, allowed_request_methods=["POST"], **kwargs)

    ## @brief Process a http POST request
    # @param received_data flask.request with a .json attribute, optionally containing a frequency and count, with both default to 1
    # @returns a http 201 response
    def post(self, received_data):
        led_service = dbusif.RemoteObject("led")

        try:
            frequency = float(received_data.json.get("frequency", 1.0)) if received_data.json else 1.0
        except ValueError:
            return FlaskResponses.createFailed("Frequency must be positive number")

        try:
            count = int(received_data.json.get("count", 1)) if received_data.json else 1
        except ValueError:
            return FlaskResponses.createFailed("count must be positive integer")

        if frequency <= 0:
            return FlaskResponses.createFailed("frequency must be > 0")

        if count <= 0:
            return FlaskResponses.createFailed("count must be > 0")

        try:
            led_service.blinkMainLighting(frequency, count)
            return FlaskResponses.createCreated("Blink set")
        except Exception as e:
            log.exception("Could not start blink")
            return FlaskResponses.createFailed("Could not start blink")
