import logging

from griffin import dbusif
from griffin.interface.http.exposedItems.httpExposedObject import HttpExposedObject
from griffin.interface.http.flaskResponses import FlaskResponses

log = logging.getLogger(__name__.split(".")[-1])


# @brief Handles the showing of a message on the printer
class MessageScreen(HttpExposedObject):
    # @brief Initializes the message screen endpoint
    # @param message_service The instance of the message service object
    def __init__(self, *args, message_service, **kwargs):
        super().__init__(*args, allowed_request_methods=["PUT"], **kwargs)
        self.__printer_service = dbusif.RemoteObject("printer")
        self.__message_service = message_service

    def put(self, received_data):
        if self.__printer_service.getProperty("state") != "idle":
            return FlaskResponses.createMethodNotAllowed()

        if received_data is None or "message" not in received_data:
            return FlaskResponses.createFailed("A message is required")

        message = str(received_data["message"])
        button_caption = "CONTINUE"

        if "button_caption" in received_data:
            button_caption = str(received_data["button_caption"])

        if message == "":
            self.__message_service.clearDisplayMessage()
        else:
            self.__message_service.setDisplayMessage(message, button_caption)

        return FlaskResponses.createSuccess()
