import logging
import os
import werkzeug

from griffin import dbusif
from griffin.interface.http.exposedItems.httpExposedObject import HttpExposedObject
from griffin.interface.http.flaskResponses import FlaskResponses

log = logging.getLogger(__name__.split(".")[-1])


# ValidateGCodeHeader is a special case of an HttpExposedObject.
class ValidateGCodeHeader(HttpExposedObject):
    ## The location where the uploaded files will be saved (tmpfs)
    __STORAGE_PATH = "/tmp/"
    ## Prefix for gcode files only used for header validation
    __PREFIX = "vh"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, allowed_request_methods=["POST"], **kwargs)
        self.__printer_service = dbusif.RemoteObject("printer")

    def post(self, received_data):
        received_file = received_data.files.get("file")
        if not received_file:
            return FlaskResponses.createFailed(message="No file received")

        log.info("Received file name %s", received_file.filename)

        ## Make sure the filename is not trying to access location on FS where it should not be
        filename = ValidateGCodeHeader.__PREFIX + werkzeug.secure_filename(received_file.filename)
        file_path = os.path.join(ValidateGCodeHeader.__STORAGE_PATH, filename)

        try:
            received_file.save(file_path)
            log.info("File saved as %s", file_path)

            result = self.__printer_service.validateHeader(file_path)
            os.remove(file_path)

            return FlaskResponses.createSuccess(json=result)

        except OSError:
            log.error("Unable to write/remove file %s", file_path)
            return FlaskResponses.createFailed(message="Unable to write/remove file %s" % file_path)