import datetime

from griffin.interface.http.exposedItems.httpExposedType import HttpExposedType
from griffin.interface.http.flaskResponses import FlaskResponses

import logging
log = logging.getLogger(__name__.split(".")[-1])


## @brief HttpExposedDatetime is a special case of an HttpExposedType, returning a ISO 8601 formatted datetime
#   As the get function, it expects the result in a timestamp (float/int) or as a datetime object.
class HttpExposedDatetime(HttpExposedType):

    ## @brief Initialize a new HttpExposedDatetime
    def __init__(self, *args, **kwargs):
        super().__init__(datetime.datetime, *args, **kwargs)

    def get(self):
        data = self._get_function()

        converted = ""
        try:
            if isinstance(data, int) or isinstance(data, float):
                if data == 0:
                    return FlaskResponses.createSuccess(None, json="")

                data = datetime.datetime.utcfromtimestamp(data)
            converted = data.replace(microsecond=0).isoformat()
        except (TypeError, ValueError, AttributeError):
            log.warning("Unable to convert data for %s. type(data) = %s, data = %s" % (self.getAbsolutePath(), type(data), data))

        return FlaskResponses.createSuccess(None, converted)
