from . import abstractEffect
from .. import colorTheme
from griffin.math import hsvColor
import time


## Blink effect.
#  This is a simple effect that blinks at a given frequency between off and the given color.
#  It can be set to repeat forever, or a certain amount of blinks.
#  When this effect finishes, it is guaranteed to leave the hardware at the "on" color.
class BlinkEffect(abstractEffect.AbstractEffect):
    # @param color: A HsvColor object
    # @param blink_frequency: float, frequency of the blinking.
    # @param repeat_count: int, Amount of blinks, or None to blink forever till aborted.
    def __init__(self, color, blink_frequency=1.0, repeat_count=None):
        super().__init__()
        assert isinstance(color, hsvColor.HsvColor)
        assert blink_frequency > 0.0
        assert repeat_count is None or repeat_count > 0

        self.__color = color
        self.__delay = 0.5 / blink_frequency
        self.__repeat_count = repeat_count

    ## run this effect
    def _run(self):
        while not self._abort_event.is_set():
            self._setOutput(colorTheme.BLACK)
            time.sleep(self.__delay)
            self._setOutput(self.__color)
            if self.__repeat_count is not None:
                self.__repeat_count -= 1
                if self.__repeat_count <= 0:
                    return
            time.sleep(self.__delay)
