from . import abstractEffect
from griffin.math import hsvColor
import time
import logging

log = logging.getLogger(__name__.split(".")[-1])


## The Glow effect
#  The glow effect cycles between two different colors, interpolating between these two colors at a set frequency.
#  The glow effect lasts till aborted.
class GlowEffect(abstractEffect.AbstractEffect):
    # @param first_color: HsvColor object, one of the colors in the glow effect.
    # @param second_color: HsvColor object, one of the colors in the glow effect.
    # @param frequency: float, frequency of the glow, in Hz.
    def __init__(self, first_color, second_color, frequency=1.0):
        super().__init__()
        assert isinstance(first_color, hsvColor.HsvColor)
        assert isinstance(first_color, hsvColor.HsvColor)
        self.__first_color = first_color
        self.__second_color = second_color
        self.__cycle_time = 1.0 / frequency

    ## Run the glow effect.
    #  This will try to run it on a hardware effect,
    #  As the head LEDs can run this effect on hardware to provide a smoother experience.
    def _run(self):
        if self._runHardwareEffect("glow", {"first_color": self.__first_color, "second_color": self.__second_color, "cycle_time": self.__cycle_time}):
            return

        start_time = time.monotonic()
        while not self._abort_event.is_set():
            t = time.monotonic() - start_time
            if t >= self.__cycle_time:
                t -= self.__cycle_time
                start_time += self.__cycle_time
            if t < self.__cycle_time / 2.0:
                f = t / (self.__cycle_time / 2.0)
            else:
                f = (self.__cycle_time - t) / (self.__cycle_time / 2.0)
            self._setOutput(self.__first_color.interpolate(self.__second_color, f))
