from . import abstractHardware
import logging
import os

from .drivers.linuxLEDClass import LinuxLEDClass

log = logging.getLogger(__name__.split(".")[-1])


## This class controls the LEDs that light up the ring around the front control knob.
class ButtonRingHardware(abstractHardware.AbstractHardware):
    __LEDS = ["red", "green", "blue"]
    def __init__(self):
        super().__init__()

        # HACK ideally, we will ask udev or the device tree what hardware
        # is available, but we cannot do this yet. So instead, we check
        # if the new path is available, and if not fall back to the old
        # file node. But even this new path should come from the machine itself
        # rather than hard coding it here.
        led_function = None
        if os.path.isdir("/sys/class/leds/pca963x:red:button_ring"):
            led_function = "button_ring"

        self.__leds = dict()
        for led in ButtonRingHardware.__LEDS:
            self.__leds[led] = LinuxLEDClass("pca963x", led, led_function)

        # Color balance data should come from the hardware definition as it is dependent of chosen LED.
        self.__color_balance = {"red": 1.0, "blue": 0.24, "green": 0.2}
        self.disableBlink()

    ## Disable blink
    def disableBlink(self):
        for led in ButtonRingHardware.__LEDS:
            self.__leds[led].disableBlink()

    ## Set the button ring leds to a color.
    #  @param red: float - Red output in range 0.0 to 1.0
    #  @param green: float - Green output in range 0.0 to 1.0
    #  @param blue: float - Blue output in range 0.0 to 1.0
    def setRGB(self, red, green, blue):
        # EM-1790 TODO, enable color balancing.
#        red *= self.__color_balance["red"]
#        green *= self.__color_balance["green"]
#        blue *= self.__color_balance["blue"]

        self.__leds["red"].setBrightness(red)
        self.__leds["green"].setBrightness(green)
        self.__leds["blue"].setBrightness(blue)
