from .gcodeFileHandler import GCodeFileHandler
from .gzipgcodeFileHandler import GzipGCodeFileHandler
from griffin.datatypes.gcodeMetaDataHandler import GCodeMetaDataHandler

import logging

log = logging.getLogger(__name__.split(".")[-1])


## File Handler Factory
class FileHandlerFactory():
    ## The possible factories
    __FILE_HANDLER_CLASSES = [GCodeFileHandler, GzipGCodeFileHandler]

    # @param filename The complete path to the file to handle.
    def __init__(self, filename):
        self.__file_handler = None
        self.__meta_data_handler = None

        for handler_class in self.__FILE_HANDLER_CLASSES:
            if handler_class.isSupportedFile(filename):
                self.__file_handler = handler_class(filename)
                self.__meta_data_handler = GCodeMetaDataHandler(self.__file_handler)

        ## TODO: EM-1361 Should test for supported types and better handling in case of unrecognized type
        if self.__file_handler is None:
            raise ValueError("Filename is not recognized as a supported filetype: %s" % filename)

    def getFileHandler(self):
        return self.__file_handler

    def getMetaDataHandler(self):
        return self.__meta_data_handler

    ## @brief Combines all supported filetypes from all handlers and returns the list
    #  @return Returns a list of all supported file extensions
    @classmethod
    def getSupportedFileTypes(cls):
        all_types = []
        for handler_class in cls.__FILE_HANDLER_CLASSES:
            all_types += handler_class.getSupportedFileExtensions()

        return all_types