from ._NumericItem import NumericItem
import uuid

##
# This is a RegisterItem type and is meant to access a specific type of data in the hotend eeprom.
#
# This class is meant to encode and decode the numeric version of a GUID from the eeprom data and converting it to and from a string for the user.
class GUIDItem(NumericItem):
    ##
    # constructs a GUID Item which is a type of RegisterItem, this register item always claims 128 bits aka 16 bytes, this cannot be changed.
    # \param address the address at which the string starts.
    # \param description all registers could use some description this is used for debugging mostly but is included for clarity
    def __init__(self, address, description):
        super().__init__(address, "ff" * 16, description)

    def getValueFromData(self, data):
        value = super().getValueFromData(data)
        return str(uuid.UUID(int=value))

    def setValueInData(self, data, value):
        numeric_value = int(uuid.UUID(value))
        return super().setValueInData(data, numeric_value)

