from griffin import dbusif
from griffin.printer.procedures.procedure import Procedure
import urllib.parse
import time
import logging
from griffin.printer.procedures.print.parseHeaderStep import ParseHeaderStep
import math

log = logging.getLogger(__name__.split(".")[-1])

class ParseHeaderProcedure(Procedure):
    def __init__(self, key):
        super().__init__(key, Procedure.TYPE_UNKNOWN)

        self._metadata = None
        self._last_metadata_timestamp = None

        self.__material_service = dbusif.RemoteObject("material")

    def setup(self, args):
        self._metadata = None
        self._last_metadata_timestamp = None
        self._url = None

        if "url" not in args:
            log.warning("No URL given to print procedure")
            return False
        try:
            url_data = urllib.parse.urlparse(args["url"], allow_fragments=False)
            self._url = args["url"]
        except Exception:
            log.exception("Failed to parse URL data")
            return False
        parse_header_step = ParseHeaderStep("PARSE_HEADER", self.getKey())

        parse_header_step.setUrlData(url_data)

        self.setFirstStep(parse_header_step)
        parse_header_step.stepFinished.connect(self._parse_finished)

        return True

    def _parse_finished(self):
        self._last_metadata_timestamp = int(time.monotonic())

    def setGcodeMetaData(self, metadata):
        self._metadata = metadata

    def getMetaData(self):
        metadata = {}
        if self._metadata is None or self._last_metadata_timestamp is None:
            log.info("metadata {metadata}, timestamp {timestamp}".format(metadata=str(self._metadata), timestamp=str(self._last_metadata_timestamp)))
            return metadata

        metadata.update(self._metadata.getDurationsAndEstimationMetadata())

        metadata.update({
            "last_metadata_timestamp": self._last_metadata_timestamp,
            "url": self._url # included so no listItem will display info not belonging to that item!
        })

        return  metadata
