from griffin.printer.controller import Controller
from griffin.printer.procedures.procedureStep import ProcedureStep
from griffin.math.vector3 import Vector3
from griffin.math.plane import Plane
import logging

log = logging.getLogger(__name__.split(".")[-1])


## The class calculates a plane (based on the first 3 points detected by active leveling. To be used during the rest of the leveling procedure so we can start closer to the buildplate.
class CalculatePlaneStep(ProcedureStep):
    def __init__(self, key, *, probe_hotend_storage, plane_storage, plane_storage_key, hotend_index):
        super().__init__(key)
        self.__probe_hotend_storage = probe_hotend_storage
        self.__plane_storage = plane_storage
        self.__plane_storage_key = plane_storage_key
        self.__hotend_index = hotend_index

    def run(self):
        assert(isinstance(self.__probe_hotend_storage, list))
        positions = self.__probe_hotend_storage
        assert(len(positions) == 3)

        if self.__hotend_index == 1:
            # If we are the right hotend, we correct the vertices to be in the coordinate space of the left hotend.
            positions = [position - Vector3(0, 0, Controller.getInstance().getPropertyValue("nominal_nozzle_z_offset")) for position in positions]

        # Calculate the plane trough these vertices.
        plane = Plane.fitPlane(positions[0], positions[1], positions[2])
        if plane.normal.z < 0:
            # We want all our normals to point in the same direction. We know we the plane should point upwards or downwards.
            # If it points downwards, flip the plane.
            plane = Plane(-plane.normal, -plane.distance)

        self.__plane_storage[self.__plane_storage_key] = plane
