import logging

from griffin.printer.controller import Controller
from griffin.printer.procedures.procedureStep import ProcedureStep

log = logging.getLogger(__name__.split(".")[-1])

## This class can set the temperature of N hotends to be found at runtime.
#  this is instead of the previously defined 1 hotend set only at the initial stage, it can still be set at the initial stage though
class SetHotendsTemperatureStep(ProcedureStep):
    # @param key the key that each step must have to distinguish itself to the external interfaces like the display...
    #  @param target_configuration a dictionary that should contain:
    #       a "hotend_nr" field this can be a number, a list of numbers or a callable returning either of the first 2 options.
    #       a "target" field this can be a number, a list of numbers or a callable returning either of the first 2 options.
    def __init__(self, key, target_configuration=None):
        super().__init__(key)
        self.__target_configuration = target_configuration

    def run(self):
        hotends = None
        targets = None
        if "hotend_nr" in self.__target_configuration:
            hotends = self.__target_configuration["hotend_nr"]
        if callable(hotends):
            hotends = hotends()
        if "target" in self.__target_configuration:
            targets = self.__target_configuration["target"]
        if callable(targets):
            targets = targets()

        if not isinstance(hotends, list):
            hotends = [hotends]
        if not isinstance(targets, list):
            targets = [targets for _ in hotends]

        assert len(targets) == len(hotends)
        heatable_hotends = [ Controller.getInstance().getPrintHeadController().getHotendSlot(hotend_nr) for hotend_nr in hotends ]
        for hotend, target_temperature in zip(heatable_hotends, targets):
            hotend.setPropertyValue("pre_tune_target_temperature", target_temperature)
