import logging

from griffin.printer.faultHandler import FaultHandler, FaultCode
from griffin.printer.controller import Controller
from .waitForHeatableObjectsStep import WaitForHeatableObjectsStep

log = logging.getLogger(__name__.split(".")[-1])


## Class to set a hotend temperature and wait for it to be reached.
class WaitForMultipleHotendsTemperatureStep(WaitForHeatableObjectsStep):
    ## Seconds to wait for Hotend to reach temperature setpoint
    __DEFAULT_HOTEND_TEMP_TIMEOUT = 10 * 60

    ## Create a new WaitForHotendTemperatureStep.
    #  @param key: string, name of this step.
    #  @param targets:
    def __init__(self, key, targets):
        super().__init__(key, None, step_timeout=self.__DEFAULT_HOTEND_TEMP_TIMEOUT)

        controller = Controller.getInstance()
        if targets == "all":
            targets = list(range(controller.getPropertyValue("hotend_count")))
        assert isinstance(targets, list)
        self.__targets = targets
        self._setHeatableObjects([controller.getPrintHeadController().getHotendSlot(hotend_index) for hotend_index in targets])

    def _onStepTimeout(self):
        FaultHandler.critical("Waiting for hotend temperature is taking too long", FaultCode.HOTEND_TEMPERATURE_TIMEOUT, data={"display_hotend_nr": "-".join(map(lambda n: str(n + 1), self.__targets))})  # For displaying
