import re
import logging
from griffin.printer.procedures.procedureStep import ProcedureStep
from griffin.printer.controller import Controller
log = logging.getLogger(__name__.split(".")[-1])


class FindSwitchingBayOffsetStep(ProcedureStep):
    def __init__(self, key):
        super().__init__(key)

    def run(self):
        c = Controller.getInstance()
        c.setGCodeActiveHotend(0)
        offX = self.__findOffset("X")
        offY = self.__findOffset("Y")
        if offX is None or offY is None:
            raise ValueError("could not parse one of the found offsets!")
        c.setPreference("SWITCH_OFFSET", [offX, offY])

    def __findOffset(self, axis):
        c = Controller.getInstance()
        data = c.sendRawGCode("G28 %s0" % axis, wait=True)
        m = re.match(r"home_distance = (\d+\.\d+)", data)
        if not m:
            return None
        if axis == "X":
            return self.__x(float(m.group(1)))
        if axis == "Y":
            return self.__y(float(m.group(1)))

    ## @brief The 0.5 is extra move into the slider block creating some pressure to keep the lifting switch in the switching bay.
    #         The rest is moving the model I got the move coordinates from to the position the switching bay was in my test-machine.
    #  @param v The raw value as result from G28 for the homing distance
    #  @return The adjusted value for the X offset
    def __x(self, v):
        bv_min_x = Controller.getInstance().getPropertyValue("movement_volume")["min"]["x"]
        return bv_min_x + v + 0.5 - (223.821600 - 119.7)  # These magic values somehow relate to the switching bay model in SwitchActiveHotendStep

    ## @brief Here no extra offset is used/applied.
    #  @param v The raw value as result from G28 for the homing distance
    #  @return The adjusted value for the Y offset
    def __y(self, v):
        bv_max_y = Controller.getInstance().getPropertyValue("movement_volume")["max"]["y"]
        return bv_max_y - v + 0 - (219.333910 - 122.6)  # These magic values somehow relate to the switching bay model in SwitchActiveHotendStep



