from griffin.printer.procedures.procedureStep import ProcedureStep
from griffin.thread import Event
from griffin.printer.controller import Controller
import logging

log = logging.getLogger(__name__.split(".")[-1])


class WaitMessageStep(ProcedureStep):

    ## @brief Initialize the WaitMessageStep
    #  @param key The step key for identification
    #  @param message The message to wait for
    #  @param user_interaction A flag to use whether or not user interaction (led ring) should be made active
    #  @param alternate_outcome_message A list of possible outcomes
    def __init__(self, key, message, *, user_interaction=False, alternate_outcome_messages=None):
        super().__init__(key, outcomes=alternate_outcome_messages)

        self.__message = message
        self.__message_received_event = Event()
        self.__user_interaction = user_interaction
        self.__alternate_outcome_messages = alternate_outcome_messages
        self.__alternate_message_received = False

    # Do nothing when this step is started. Just wait for the proper message to be received.
    def run(self):
        self.__alternate_message_received = False
        self.__message_received_event.clear()
        Controller.getInstance().setPropertyValue("interaction_required", self.__user_interaction)

        while not self.__message_received_event.is_set() and not self._abort:
            self.__message_received_event.wait(self.TIME_OUT_WAIT_FINISHED_EVENT)

        Controller.getInstance().setPropertyValue("interaction_required", False)

        if self.__alternate_message_received:
            return self.__alternate_message_received

    def receiveMessage(self, message):
        log.info("Message: %s", message)

        if message == self.__message:
            self.__message_received_event.set()
        elif self.__alternate_outcome_messages is not None and message in self.__alternate_outcome_messages:
            self.__alternate_message_received = message
            self.__message_received_event.set()
        else:
            super().receiveMessage(message)